package proxy.command;

import com.google.common.collect.Collections2;

import proxy.Proxy;
import proxy.handler.ProxyHandler;
import proxy.util.Formatter;
import proxy.util.User;

public class CommandAdmin extends Command {
	public String getName() {
		return "admin";
	}
	
	public String getHelp() {
		return "Makes a user an admin or lists admins";
	}
	
	public String getArgs() {
		return "[username]";
	}
	
	public String getPermission() {
		return "";
	}

	public void run(Proxy proxy, ProxyHandler player, String[] args) {
		if(args.length == 0) {
			int cnt = 0;
			for(User user : proxy.getUsers()) {
				if(user.isAdmin()) {
					sendMessage(player, Formatter.GREEN + user.getUsername());
					cnt++;
				}
			}
			sendMessage(player, (cnt == 0 ? Formatter.GRAY + "No admins" : Formatter.DARK_GREEN + (cnt == 1 ? "1 admin" : "%d admins")) + " on this proxy", cnt);
			return;
		}
		User user = proxy.getUser(args[0]);
		if(user == null)
			throw new RunException("'%s' is not registered", args[0]);
		if(user.isAdmin())
			throw new RunException("%s is already an admin", user.getUsername());
		user.setAdmin(true);
		if(user.isOnline())
			((ProxyHandler)user).sendMessage(Formatter.GOLD + "You were given proxy admin privileges");
		sendInfo(player, "%s is now an admin", user.getUsername());
	}
	
	public Iterable<String> complete(Proxy proxy, ProxyHandler player, String[] args, String[] serverMatches) {
		return args.length == 1 ? Collections2.filter(proxy.getUserNames(), user -> !proxy.getUser(user).isAdmin()) : null;
	}
}
