package proxy.command;

import com.google.common.collect.Collections2;

import proxy.Proxy;
import proxy.handler.ProxyHandler;
import proxy.util.User;

public class CommandDelete extends Command {
	public String getName() {
		return "delete";
	}
	
	public String getHelp() {
		return "Deletes a users profile (console only for admins)";
	}
	
	public String getArgs() {
		return "<username>";
	}

	public void run(Proxy proxy, ProxyHandler player, String[] args) {
		if(args.length < 1)
			throw new RunException("Please provide a username");
		User user = proxy.getUser(args[0]);
		if(user == null)
			throw new RunException("'%s' is not registered", args[0]);
		if(player != null && user.isAdmin())
			throw new RunException("Only the console can delete users with admin status");
		if(user.isOnline())
			((ProxyHandler)user).disconnect("Your profile was deleted");
		proxy.deleteUser(user.getUsername());
		sendInfo(player, "The profile of %s was deleted", user.getUsername());
	}
	
	public Iterable<String> complete(Proxy proxy, ProxyHandler player, String[] args, String[] serverMatches) {
		return args.length == 1 ? (player == null ? proxy.getUserNames() : Collections2.filter(proxy.getUserNames(), user -> !proxy.getUser(user).isAdmin())) : null;
	}
}
