package proxy.command;

import com.google.common.collect.Collections2;

import proxy.Proxy;
import proxy.handler.ProxyHandler;
import proxy.util.Formatter;

public class CommandKick extends Command {
	public String getName() {
		return "kick";
	}
	
	public String getHelp() {
		return "Kicks a player off the server";
	}
	
	public String getArgs() {
		return "<player> [reason ...]";
	}

	public void run(Proxy proxy, ProxyHandler player, String[] args) {
		if(args.length < 1)
			throw new RunException("Please provide a player name");
		ProxyHandler plr = proxy.getPlayer(args[0]);
		if(plr == null)
			throw new RunException("'%s' is not online", args[0]);
		if(plr == player)
			throw new RunException("You cannot kick yourself");
		if(player != null && plr.isAdmin())
			throw new RunException("Admins can only be kicked by the console");
		plr.disconnect(args.length <= 1 ? "Kicked from server" : Formatter.joinSpace(1, args));
	}
	
	public Iterable<String> complete(Proxy proxy, ProxyHandler player, String[] args, String[] serverMatches) {
		return args.length == 1 ? (player == null ? proxy.getPlayerNames() : Collections2.filter(proxy.getPlayerNames(), name -> !proxy.getPlayer(name).isAdmin())) : null;
	}
}
