package proxy.network;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;
import com.google.common.collect.Maps;

import proxy.packet.*;
import proxy.util.Log;

public enum Protocol
{
    HANDSHAKE(-1)
    {
        {
            this.registerPacket(Direction.SERVER, H00PacketHandshake.class);
        }
    },
    PLAY(0)
    {
        {
            this.registerPacket(Direction.CLIENT, S00PacketKeepAlive.class);
            this.registerPacket(Direction.CLIENT, S01PacketJoinGame.class);
            this.registerPacket(Direction.CLIENT, S02PacketChat.class);
            this.registerPacket(Direction.CLIENT, S03PacketTimeUpdate.class);
            this.registerPacket(Direction.CLIENT, S04PacketEntityEquipment.class);
            this.registerPacket(Direction.CLIENT, S05PacketSpawnPosition.class);
            this.registerPacket(Direction.CLIENT, S06PacketUpdateHealth.class);
            this.registerPacket(Direction.CLIENT, S07PacketRespawn.class);
            this.registerPacket(Direction.CLIENT, S08PacketPlayerPosLook.class);
            this.registerPacket(Direction.CLIENT, S09PacketHeldItemChange.class);
            this.registerPacket(Direction.CLIENT, S0APacketUseBed.class);
            this.registerPacket(Direction.CLIENT, S0BPacketAnimation.class);
            this.registerPacket(Direction.CLIENT, S0CPacketSpawnPlayer.class);
            this.registerPacket(Direction.CLIENT, S0DPacketCollectItem.class);
            this.registerPacket(Direction.CLIENT, S0EPacketSpawnObject.class);
            this.registerPacket(Direction.CLIENT, S0FPacketSpawnMob.class);
            this.registerPacket(Direction.CLIENT, S10PacketSpawnPainting.class);
            this.registerPacket(Direction.CLIENT, S11PacketSpawnExperienceOrb.class);
            this.registerPacket(Direction.CLIENT, S12PacketEntityVelocity.class);
            this.registerPacket(Direction.CLIENT, S13PacketDestroyEntities.class);
            this.registerPacket(Direction.CLIENT, S14PacketEntity.class);
            this.registerPacket(Direction.CLIENT, S14PacketEntity.S15PacketEntityRelMove.class);
            this.registerPacket(Direction.CLIENT, S14PacketEntity.S16PacketEntityLook.class);
            this.registerPacket(Direction.CLIENT, S14PacketEntity.S17PacketEntityLookMove.class);
            this.registerPacket(Direction.CLIENT, S18PacketEntityTeleport.class);
            this.registerPacket(Direction.CLIENT, S19PacketEntityHeadLook.class);
            this.registerPacket(Direction.CLIENT, S19PacketEntityStatus.class);
            this.registerPacket(Direction.CLIENT, S1BPacketEntityAttach.class);
            this.registerPacket(Direction.CLIENT, S1CPacketEntityMetadata.class);
            this.registerPacket(Direction.CLIENT, S1DPacketEntityEffect.class);
            this.registerPacket(Direction.CLIENT, S1EPacketRemoveEntityEffect.class);
            this.registerPacket(Direction.CLIENT, S1FPacketSetExperience.class);
            this.registerPacket(Direction.CLIENT, S20PacketEntityProperties.class);
            this.registerPacket(Direction.CLIENT, S21PacketChunkData.class);
            this.registerPacket(Direction.CLIENT, S22PacketMultiBlockChange.class);
            this.registerPacket(Direction.CLIENT, S23PacketBlockChange.class);
            this.registerPacket(Direction.CLIENT, S24PacketBlockAction.class);
            this.registerPacket(Direction.CLIENT, S25PacketBlockBreakAnim.class);
            this.registerPacket(Direction.CLIENT, S26PacketMapChunkBulk.class);
            this.registerPacket(Direction.CLIENT, S27PacketExplosion.class);
            this.registerPacket(Direction.CLIENT, S28PacketEffect.class);
            this.registerPacket(Direction.CLIENT, S29PacketSoundEffect.class);
            this.registerPacket(Direction.CLIENT, S2APacketParticles.class);
            this.registerPacket(Direction.CLIENT, S2BPacketChangeGameState.class);
            this.registerPacket(Direction.CLIENT, S2CPacketSpawnGlobalEntity.class);
            this.registerPacket(Direction.CLIENT, S2DPacketOpenWindow.class);
            this.registerPacket(Direction.CLIENT, S2EPacketCloseWindow.class);
            this.registerPacket(Direction.CLIENT, S2FPacketSetSlot.class);
            this.registerPacket(Direction.CLIENT, S30PacketWindowItems.class);
            this.registerPacket(Direction.CLIENT, S31PacketWindowProperty.class);
            this.registerPacket(Direction.CLIENT, S32PacketConfirmTransaction.class);
            this.registerPacket(Direction.CLIENT, S33PacketUpdateSign.class);
            this.registerPacket(Direction.CLIENT, S34PacketMaps.class);
            this.registerPacket(Direction.CLIENT, S35PacketUpdateTileEntity.class);
            this.registerPacket(Direction.CLIENT, S36PacketSignEditorOpen.class);
            this.registerPacket(Direction.CLIENT, S37PacketStatistics.class);
            this.registerPacket(Direction.CLIENT, S38PacketPlayerListItem.class);
            this.registerPacket(Direction.CLIENT, S39PacketPlayerAbilities.class);
            this.registerPacket(Direction.CLIENT, S3APacketTabComplete.class);
            this.registerPacket(Direction.CLIENT, S3BPacketScoreboardObjective.class);
            this.registerPacket(Direction.CLIENT, S3CPacketUpdateScore.class);
            this.registerPacket(Direction.CLIENT, S3DPacketDisplayScoreboard.class);
            this.registerPacket(Direction.CLIENT, S3EPacketTeams.class);
            this.registerPacket(Direction.CLIENT, S3FPacketCustomPayload.class);
            this.registerPacket(Direction.CLIENT, S40PacketDisconnect.class);
            this.registerPacket(Direction.CLIENT, S41PacketServerDifficulty.class);
            this.registerPacket(Direction.CLIENT, S42PacketCombatEvent.class);
            this.registerPacket(Direction.CLIENT, S43PacketCamera.class);
            this.registerPacket(Direction.CLIENT, S44PacketWorldBorder.class);
            this.registerPacket(Direction.CLIENT, S45PacketTitle.class);
            this.registerPacket(Direction.CLIENT, S46PacketSetCompressionLevel.class);
            this.registerPacket(Direction.CLIENT, S47PacketPlayerListHeaderFooter.class);
            this.registerPacket(Direction.CLIENT, S48PacketResourcePackSend.class);
            this.registerPacket(Direction.CLIENT, S49PacketUpdateEntityNBT.class);
            this.registerPacket(Direction.SERVER, C00PacketKeepAlive.class);
            this.registerPacket(Direction.SERVER, C01PacketChatMessage.class);
            this.registerPacket(Direction.SERVER, C02PacketUseEntity.class);
            this.registerPacket(Direction.SERVER, C03PacketPlayer.class);
            this.registerPacket(Direction.SERVER, C03PacketPlayer.C04PacketPlayerPosition.class);
            this.registerPacket(Direction.SERVER, C03PacketPlayer.C05PacketPlayerLook.class);
            this.registerPacket(Direction.SERVER, C03PacketPlayer.C06PacketPlayerPosLook.class);
            this.registerPacket(Direction.SERVER, C07PacketPlayerDigging.class);
            this.registerPacket(Direction.SERVER, C08PacketPlayerBlockPlacement.class);
            this.registerPacket(Direction.SERVER, C09PacketHeldItemChange.class);
            this.registerPacket(Direction.SERVER, C0APacketAnimation.class);
            this.registerPacket(Direction.SERVER, C0BPacketEntityAction.class);
            this.registerPacket(Direction.SERVER, C0CPacketInput.class);
            this.registerPacket(Direction.SERVER, C0DPacketCloseWindow.class);
            this.registerPacket(Direction.SERVER, C0EPacketClickWindow.class);
            this.registerPacket(Direction.SERVER, C0FPacketConfirmTransaction.class);
            this.registerPacket(Direction.SERVER, C10PacketCreativeInventoryAction.class);
            this.registerPacket(Direction.SERVER, C11PacketEnchantItem.class);
            this.registerPacket(Direction.SERVER, C12PacketUpdateSign.class);
            this.registerPacket(Direction.SERVER, C13PacketPlayerAbilities.class);
            this.registerPacket(Direction.SERVER, C14PacketTabComplete.class);
            this.registerPacket(Direction.SERVER, C15PacketClientSettings.class);
            this.registerPacket(Direction.SERVER, C16PacketClientStatus.class);
            this.registerPacket(Direction.SERVER, C17PacketCustomPayload.class);
            this.registerPacket(Direction.SERVER, C18PacketSpectate.class);
            this.registerPacket(Direction.SERVER, C19PacketResourcePackStatus.class);
        }
    },
    STATUS(1)
    {
        {
            this.registerPacket(Direction.SERVER, Q00PacketServerQuery.class);
            this.registerPacket(Direction.CLIENT, I00PacketServerInfo.class);
            this.registerPacket(Direction.SERVER, Q01PacketPing.class);
            this.registerPacket(Direction.CLIENT, I01PacketPong.class);
        }
    },
    LOGIN(2)
    {
        {
            this.registerPacket(Direction.CLIENT, R00PacketDisconnect.class);
            this.registerPacket(Direction.CLIENT, R01PacketEncryptionRequest.class);
            this.registerPacket(Direction.CLIENT, R02PacketLoginSuccess.class);
            this.registerPacket(Direction.CLIENT, R03PacketEnableCompression.class);
            this.registerPacket(Direction.SERVER, L00PacketLoginStart.class);
            this.registerPacket(Direction.SERVER, L01PacketEncryptionResponse.class);
        }
    };

    private static int field_181136_e = -1;
    private static int field_181137_f = 2;
    private static final Protocol[] STATES_BY_ID = new Protocol[field_181137_f - field_181136_e + 1];
    private static final Map < Class <? extends Packet > , Protocol > STATES_BY_CLASS = Maps. < Class <? extends Packet > , Protocol > newHashMap();
    private final int id;
    private final Map < Direction, BiMap < Integer, Class <? extends Packet >>> directionMaps;

    private Protocol(int protocolId)
    {
        this.directionMaps = Maps.newEnumMap(Direction.class);
        this.id = protocolId;
    }

    protected Protocol registerPacket(Direction direction, Class <? extends Packet > packetClass)
    {
        BiMap < Integer, Class <? extends Packet >> bimap = (BiMap)this.directionMaps.get(direction);

        if (bimap == null)
        {
            bimap = HashBiMap. < Integer, Class <? extends Packet >> create();
            this.directionMaps.put(direction, bimap);
        }

        if (bimap.containsValue(packetClass))
        {
            String s = direction + " packet " + packetClass + " is already known to ID " + bimap.inverse().get(packetClass);
            Log.error(s);
            throw new IllegalArgumentException(s);
        }
        else
        {
            bimap.put(Integer.valueOf(bimap.size()), packetClass);
            return this;
        }
    }

    public Integer getPacketId(Direction direction, Packet packetIn)
    {
        return (Integer)((BiMap)this.directionMaps.get(direction)).inverse().get(packetIn.getClass());
    }

    public Packet getPacket(Direction direction, int packetId) throws InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        Class <? extends Packet > oclass = (Class)((BiMap)this.directionMaps.get(direction)).get(Integer.valueOf(packetId));
        return oclass == null ? null : (Packet)oclass.getConstructor().newInstance();
    }

    public int getId()
    {
        return this.id;
    }

    public static Protocol getById(int stateId)
    {
        return stateId >= field_181136_e && stateId <= field_181137_f ? STATES_BY_ID[stateId - field_181136_e] : null;
    }

    public static Protocol getFromPacket(Packet packetIn)
    {
        return (Protocol)STATES_BY_CLASS.get(packetIn.getClass());
    }

    static {
        for (Protocol enumconnectionstate : values())
        {
            int i = enumconnectionstate.getId();

            if (i < field_181136_e || i > field_181137_f)
            {
                throw new Error("Invalid protocol ID " + Integer.toString(i));
            }

            STATES_BY_ID[i - field_181136_e] = enumconnectionstate;

            for (Direction enumpacketdirection : enumconnectionstate.directionMaps.keySet())
            {
                for (Class <? extends Packet > oclass : (enumconnectionstate.directionMaps.get(enumpacketdirection)).values())
                {
                    if (STATES_BY_CLASS.containsKey(oclass) && STATES_BY_CLASS.get(oclass) != enumconnectionstate)
                    {
                        throw new Error("Packet " + oclass + " is already assigned to protocol " + STATES_BY_CLASS.get(oclass) + " - can\'t reassign to " + enumconnectionstate);
                    }

                    try
                    {
                        oclass.getConstructor().newInstance();
                    }
                    catch (Throwable var10)
                    {
                        throw new Error("Packet " + oclass + " fails instantiation checks! " + oclass);
                    }

                    STATES_BY_CLASS.put(oclass, enumconnectionstate);
                }
            }
        }
    }
}
