package proxy.packet;

import java.io.IOException;

import proxy.handler.ProxyHandler;
import proxy.network.Packet;
import proxy.network.PacketBuffer;

public class C08PacketPlayerBlockPlacement implements Packet<ProxyHandler>
{
    private Vec3 position;
    private int placedBlockDirection;
    private Stack stack;
    private float facingX;
    private float facingY;
    private float facingZ;

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.position = buf.readVector();
        this.placedBlockDirection = buf.readUnsignedByte();
        this.stack = buf.readStack();
        this.facingX = (float)buf.readUnsignedByte() / 16.0F;
        this.facingY = (float)buf.readUnsignedByte() / 16.0F;
        this.facingZ = (float)buf.readUnsignedByte() / 16.0F;
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeVector(this.position);
        buf.writeByte(this.placedBlockDirection);
        buf.writeStack(this.stack);
        buf.writeByte((int)(this.facingX * 16.0F));
        buf.writeByte((int)(this.facingY * 16.0F));
        buf.writeByte((int)(this.facingZ * 16.0F));
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(ProxyHandler handler)
    {
        handler.processPlayerBlockPlacement(this);
    }
}
