package proxy.packet;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

public abstract class Nbt {
	private static Object read(byte id, DataInput input, int depth) throws IOException {
		switch(id) {
			case 0:
				return null;
			case 1:
				return input.readByte();
			case 2:
				return input.readShort();
			case 3:
				return input.readInt();
			case 4:
				return input.readLong();
			case 5:
				return input.readFloat();
			case 6:
				return input.readDouble();
			case 7: {
				int len = input.readInt();
				byte[] data = new byte[len];
				input.readFully(data);
				return data;
			}
			case 8:
				return input.readUTF();
			case 9: {
				if(depth > 512)
					throw new RuntimeException("Tried to read tag tree deeper than 512");
				byte type = input.readByte();
				int len = input.readInt();
				if(type == 0 && len > 0)
					throw new RuntimeException("Type null on list tag");
				List<Object> list = Lists.newArrayListWithCapacity(len);
				for(int z = 0; z < len; z++) {
					list.add(read(type, input, depth + 1));
				}
				return list;
			}
			case 10:
				if(depth > 512)
					throw new RuntimeException("Tried to read tag tree deeper than 512");
				Map<String, Object> tags = Maps.newHashMap();
				byte type;
				while((type = input.readByte()) != 0) {
					String key = input.readUTF();
					tags.put(key, read(type, input, depth + 1));
				}
				return tags;
			case 11: {
				int len = input.readInt();
				int[] data = new int[len];
				for(int z = 0; z < len; z++) {
					data[z] = input.readInt();
				}
				return data;
			}
			default:
				return null;
		}
	}
	
	private static byte getId(Object obj) {
		return (byte)(obj instanceof Byte ? 1 : (obj instanceof Short ? 2 : (obj instanceof Integer ? 3 : (obj instanceof Long ? 4 : (obj instanceof Float ? 5 : (obj instanceof Double ? 6 : (obj instanceof byte[] ? 7 : (obj instanceof String ? 8 : (obj instanceof List ? 9 : (obj instanceof Map ? 10 : (obj instanceof int[] ? 11 : 0)))))))))));
	}
	
	private static void write(Object obj, DataOutput output) throws IOException {
		if(obj instanceof Byte)
			output.writeByte((Byte)obj);
		else if(obj instanceof Short)
			output.writeShort((Short)obj);
		else if(obj instanceof Integer)
			output.writeInt((Integer)obj);
		else if(obj instanceof Long)
			output.writeLong((Long)obj);
		else if(obj instanceof Float)
			output.writeFloat((Float)obj);
		else if(obj instanceof Double)
			output.writeDouble((Double)obj);
		else if(obj instanceof byte[]) {
			output.writeInt(((byte[])obj).length);
			output.write(((byte[])obj));
		}
		else if(obj instanceof String)
			output.writeUTF((String)obj);
		else if(obj instanceof List) {
			output.writeByte(((List)obj).isEmpty() ? 0 : getId(((List)obj).get(0)));
			output.writeInt(((List)obj).size());
			for(int z = 0; z < ((List)obj).size(); z++) {
				write(((List)obj).get(z), output);
			}
		}
		else if(obj instanceof Map) {
			for(String key : ((Map<String, Object>)obj).keySet()) {
				Object data = ((Map<String, Object>)obj).get(key);
				output.writeByte(getId(data));
				if(data != null) {
					output.writeUTF(key);
					write(data, output);
				}
			}
			output.writeByte(0);
		}
		else if(obj instanceof int[]) {
			output.writeInt(((int[])obj).length);
			for(int z = 0; z < ((int[])obj).length; z++) {
				output.writeInt(((int[])obj)[z]);
			}
		}
	}

	public static Map<String, Object> readTag(DataInput input) throws IOException {
		if(input.readByte() != 10)
			throw new IOException("Root is not a compound");
		input.readUTF();
		return (Map<String, Object>)read((byte)10, input, 0);
	}

	public static void writeTag(Map<String, Object> tag, DataOutput output) throws IOException {
		output.writeByte(10);
		output.writeUTF("");
		write(tag, output);
	}

	public static String getString(Map<String, Object> tags, String key) {
		Object data = tags.get(key);
		return data instanceof String ? ((String)data) : null;
	}

	public static int getStringListLength(Map<String, Object> tags, String key) {
		Object data = tags.get(key);
		if(!(data instanceof List))
			return -1;
		return ((List)data).isEmpty() ? 0 : (((List)data).get(0) instanceof String ? ((List)data).size() : -1);
	}
}
