package proxy.packet;

import java.io.IOException;

import proxy.handler.ProxyHandler;
import proxy.network.Packet;
import proxy.network.PacketBuffer;
import proxy.util.Formatter;

public class S02PacketChat implements Packet<ProxyHandler>
{
    private String chatComponent;
    private byte type;

    public S02PacketChat()
    {
    }

    public S02PacketChat(String message, boolean hotbar)
    {
        this.chatComponent = Formatter.toJsonString(message);
        this.type = (byte)(hotbar ? 2 : 0);
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.chatComponent = buf.readChatComponent();
        this.type = buf.readByte();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeChatComponent(this.chatComponent);
        buf.writeByte(this.type);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(ProxyHandler handler)
    {
        handler.handleChat(this);
    }
    
    public String getMessage() {
    	return Formatter.getStringOrJson(this.chatComponent);
    }
    
    public byte getType() {
    	return this.type;
    }
}
