package proxy.packet;

import com.google.common.collect.Lists;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;

import proxy.Proxy;

import java.util.List;
import java.util.Random;

public class ServerInfo {
	private static final Gson GSON = new GsonBuilder().create();
	private static final Random RANDOM = new Random();
	
	private final String name;
	private final int protocol;
	private final List<String> motds = Lists.newArrayList();
	private final List<String> list = Lists.newArrayList();
	private final List<String> icons = Lists.newArrayList();
	
	private int capacity = 0;
	private int online = 0;

	public ServerInfo(String name, int protocol) {
		this.name = name;
		this.protocol = protocol;
	}

	public List<String> getMotds() {
		return this.motds;
	}

	public List<String> getIcons() {
		return this.icons;
	}

	public List<String> getList() {
		return this.list;
	}

	public void setCapacity(int capacity) {
		this.capacity = capacity;
	}

	public void setOnline(int online) {
		this.online = online;
	}

	public String serialize() {
		try {
			JsonObject obj = new JsonObject();
			obj.addProperty("description", this.motds.isEmpty() ? "" : this.motds.get(RANDOM.nextInt(this.motds.size())));
			JsonObject info = new JsonObject();
			info.addProperty("max", this.capacity);
			info.addProperty("online", this.online);
			if(!this.list.isEmpty()) {
				JsonArray arr = new JsonArray();
				for(int z = 0; z < this.list.size(); z++) {
					String line = this.list.get(z);
					JsonObject profile = new JsonObject();
					profile.addProperty("id", Proxy.getOfflineUUID(line).toString());
					profile.addProperty("name", line);
					arr.add(profile);
				}
				info.add("sample", arr);
			}
			obj.add("players", info);
			JsonObject version = new JsonObject();
			version.addProperty("name", this.name);
			version.addProperty("protocol", this.protocol);
			obj.add("version", version);
			if(!this.icons.isEmpty())
				obj.addProperty("favicon", this.icons.get(RANDOM.nextInt(this.icons.size())));
			return GSON.toJson(obj);
		}
		catch(Throwable t) {
			return "{}";
		}
	}
}
