package proxy.util;

import java.util.regex.Pattern;

import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.google.gson.JsonPrimitive;

public enum Formatter {
	BLACK('0'),
	DARK_BLUE('1'),
	DARK_GREEN('2'),
	DARK_AQUA('3'),
	DARK_RED('4'),
	DARK_PURPLE('5'),
	GOLD('6'),
	GRAY('7'),
	DARK_GRAY('8'),
	BLUE('9'),
	GREEN('a'),
	AQUA('b'),
	RED('c'),
	LIGHT_PURPLE('d'),
	YELLOW('e'),
	WHITE('f'),
	OBFUSCATED('k'),
	BOLD('l'),
	STRIKETHROUGH('m'),
	UNDERLINE('n'),
	ITALIC('o'),
	RESET('r');

	private static final Pattern STRIP_PATTERN = Pattern.compile("(?i)" + String.valueOf('\u00a7') + "[0-9A-FK-OR]");

	private final String value;

	public static String strip(String str) {
		return str == null ? null : STRIP_PATTERN.matcher(str).replaceAll("");
	}
	
	public static String toJsonString(String text) {
		return new JsonPrimitive(text).toString();
	}
	
	private static String fromJsonString(String json, boolean object) {
		JsonParser parser = new JsonParser();
		JsonElement elem;
		try {
			elem = parser.parse(json);
		}
		catch(Throwable t) {
			return null;
		}
		if(object && elem.isJsonObject())
			return elem.getAsJsonObject().has("text") && elem.getAsJsonObject().get("text").isJsonPrimitive() ? elem.getAsJsonObject().get("text").getAsString() : null;
		return elem.isJsonPrimitive() ? elem.getAsString() : null;
	}
	
	public static String fromJsonString(String json) {
		return fromJsonString(json, true);
	}
	
	public static String getStringOrJson(String json) {
		String str = fromJsonString(json, false);
		return str == null ? "JSON:" + json : str;
	}
	
	public static String filterSpaces(String str) {
		StringBuilder sb = new StringBuilder();
		boolean space = true;
		for(int z = 0; z < str.length(); z++) {
			char ch = str.charAt(z);
			if(ch != ' ') {
				sb.append(ch);
				space = false;
			}
			else if(!space) {
				sb.append(' ');
				space = true;
			}
		}
		return sb.toString().trim();
	}
	
	public static String joinSpace(int index, String ... objs) {
		return joinSpace(index, (Object[])objs);
	}
	
	public static String joinSpace(int index, Object ... objs) {
		StringBuilder sb = new StringBuilder();
		for(int z = index; z < objs.length; z++) {
			if(sb.length() != 0)
				sb.append(' ');
			sb.append(objs[z]);
		}
		return sb.toString();
	}

	public static boolean isValidUser(String name) {
		for(int z = 0; z < name.length(); z++) {
			char ch = name.charAt(z);
			if((ch < '0' || ch > '9') && (ch < 'a' || ch > 'z') && (ch < 'A' || ch > 'Z') && ch != '_')
				return false;
		}
		return true;
	}

	public static String getHexString(byte[] bytes) {
		StringBuilder sb = new StringBuilder();
		for(int z = 0; z < bytes.length; z++) {
			sb.append(String.format("%02x", bytes[z]));
		}
		return sb.toString();
	}

	public static byte[] fromHexString(String str) {
		if((str.length() & 1) == 1)
			str = "0" + str;
		byte[] bytes = new byte[str.length() / 2];
		try {
			for(int z = 0; z < bytes.length; z++) {
				bytes[z] = (byte)Integer.parseUnsignedInt(str.substring(z * 2, (z + 1) * 2), 16);
			}
		}
		catch(NumberFormatException e) {
			return null;
		}
		return bytes;
	}

	private Formatter(char code) {
		this.value = "\u00a7" + code;
	}

	public String toString() {
		return this.value;
	}
}
