package proxy.util;

import java.lang.reflect.Field;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonPrimitive;

public class Option {
	private final Field field;
	private final Object object;
	
	public Option(Field field, Object object) {
		this.field = field;
		if(this.field.getType() != int.class && this.field.getType() != boolean.class && this.field.getType() != String.class && this.field.getType() != String[].class)
			throw new IllegalArgumentException(field.getName() + ": is not int, boolean, String or String[]");
		this.object = object;
	}
	
	public JsonElement get() {
		Object obj;
		try {
			obj = this.field.get(this.object);
		}
		catch(IllegalArgumentException | IllegalAccessException e) {
			throw new RuntimeException(e);
		}
		if(this.field.getType() == int.class)
			return new JsonPrimitive((Integer)obj);
		if(this.field.getType() == boolean.class)
			return new JsonPrimitive((Boolean)obj);
		if(this.field.getType() == String.class)
			return new JsonPrimitive((String)obj);
		JsonArray arr = new JsonArray();
		for(String str : (String[])obj) {
			arr.add(new JsonPrimitive(str));
		}
		return arr;
	}
	
	public boolean set(JsonElement data) {
		Object value;
		if(this.field.getType() == int.class) {
			if(!data.isJsonPrimitive())
				return false;
			value = data.getAsNumber().intValue();
		}
		else if(this.field.getType() == boolean.class) {
			if(!data.isJsonPrimitive())
				return false;
			value = data.getAsBoolean();
		}
		else if(this.field.getType() == String.class) {
			if(!data.isJsonPrimitive())
				return false;
			value = data.getAsString();
		}
		else {
			if(data.isJsonPrimitive()) {
				value = data.getAsString();
			}
			else {
				if(!data.isJsonArray())
					return false;
				JsonArray arr = data.getAsJsonArray();
				String[] strs = new String[arr.size()];
				for(int z = 0; z < strs.length; z++) {
					if(!arr.get(z).isJsonPrimitive())
						return false;
					strs[z] = arr.get(z).getAsString();
				}
				value = strs;
			}
		}
		try {
			this.field.set(this.object, value);
		}
		catch(IllegalArgumentException | IllegalAccessException e) {
			throw new RuntimeException(e);
		}
		return true;
	}
}
