package proxy.util;

import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.lang.ProcessBuilder.Redirect;
import java.util.Locale;

import proxy.Proxy;

public class ViaProxyProcess {
	private final Proxy proxy;
	
	private Process process;
	private PrintWriter writer;
	
	public ViaProxyProcess(Proxy proxy) {
		this.proxy = proxy;
	}
	
	private static String getJavaDir() {
		String separator = System.getProperty("file.separator");
		String path = System.getProperty("java.home") + separator + "bin" + separator;
		return System.getProperty("os.name").toLowerCase(Locale.US).contains("win") && (new File(path + "javaw.exe")).isFile() ? path + "javaw.exe" : path + "java";
	}
	
	public boolean start() {
		if(!new File(this.proxy.getViaProxyJar()).exists()) {
			Log.error("The file %s could not be found in the current directory. Please download the lastest Version of ViaProxy and place it at %s.", 
				this.proxy.getViaProxyJar(), new File(this.proxy.getViaProxyJar()).getAbsolutePath());
			return false;
		}
		try {
			this.process = new ProcessBuilder(
				this.proxy.getViaProxyJavaPath().isEmpty() ? getJavaDir() : this.proxy.getViaProxyJavaPath(), 
				"-Xmx" + this.proxy.getViaProxyMaxMem() + "m", 
				"-jar", this.proxy.getViaProxyJar(), 
				"cli", 
				"--proxy-online-mode", "false", 
				"--chat-signing", "false", 
				"--target-version", "1.8.x", 
				"--compression-threshold", "" + this.proxy.getCompression(), 
				"--target-address", "127.0.0.1:" + this.proxy.getProxyPort(), 
				"--bind-address", (this.proxy.getBindHost().isEmpty() ? "0.0.0.0" : this.proxy.getBindHost()) + ":" + this.proxy.getBindPort()
			).redirectOutput(Redirect.INHERIT).redirectError(Redirect.INHERIT).redirectInput(Redirect.PIPE).start();
		}
		catch(IOException e) {
			Log.error(e, "Could not start ViaProxy process");
			return false;
		}
		this.writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new BufferedOutputStream(this.process.getOutputStream()))), true);
		return true;
	}
	
	public void shutdown() {
		if(this.writer != null) {
			try {
				this.writer.println("exit");
			}
			catch(Exception e) {
			}
		}
		if(this.process != null) {
			try {
				this.process.waitFor();
			}
			catch(InterruptedException e) {
				Log.error(e, "Interrupted while waiting for ViaProxy to terminate");
			}
		}
	}
	
	public void runCommand(String cmd) {
		if(this.writer != null) {
			try {
				this.writer.println(cmd);
			}
			catch(Exception e) {
			}
		}
	}
}
